/**
 * js logic specific to opening and closing the accordion items
 */

import ee from "~/event-emitter";
import { toArray } from "~/dom-methods";

const accordionControlSelector = ".c-control--accordion";
const accordionSelector = ".c-accordion";
const accordionOpenClass = "c-accordion__item--open";
const hideArrowsIfNoChildren = true;

let accordionControl;
let selectedItem;

/**
 * @method init
 * @description init method, adds event listener for scroll and resize
 * @return {void}
 */
export default function init() {
  accordionControl = toArray(document.querySelectorAll(accordionControlSelector));

  accordionControl.forEach((el) => {
    if (hideArrowsIfNoChildren) {
      const accordionItem = el.parentElement.parentElement;
      const subAccordionList = accordionItem.querySelector(accordionSelector);
      const hasSubAccordion = subAccordionList !== null;
      if (!hasSubAccordion) {
        accordionItem.classList.add("c-accordion__item--empty");
      }
    }
  });

  const accordionContainer = document.querySelector(".c-accordion-top-level");
  accordionContainer.addEventListener("click", onAccordionContainerClicked);

  ee.on("accordion:clicked", () => {
    resetContainerHeight();
  });
}

/**
 * @method onAccordionContainerClicked
 * @description callback for clicking on the accordion container
 * @param e {MouseEvent} click event
 * @return {void}
 */
export function onAccordionContainerClicked(e) {
  const targetElement = e.target.parentElement;
  const isAccordionButton = targetElement.classList.contains("c-control--accordion");
  isAccordionButton && onAccordionClicked(targetElement, e);
}

/**
 * @method closeAllItems
 * @description removes all open classes
 * @return {void}
 */
export function closeAllItems() {
  const accordionItem = toArray(document.querySelectorAll(".c-accordion__item"));

  accordionItem.forEach((el) => {

    // only close accordion items that is not currently in focus
    el !== selectedItem && el.classList.remove(accordionOpenClass);
  });
}

/**
 * @method isRootAccordionItem
 * @description if element item contains an accordion level 2 return true
 * @param accordionItem {HTMLElement}
 * @return {boolean}
 */
export function isRootAccordionItem(accordionItem) {
  return accordionItem.querySelector(".c-accordion--level-2") !== null;
}

/**
 * @method resetContainerHeight
 * @description calculate the height of the ul and set that to the container
 * @return {void}
 */
export function resetContainerHeight() {
  const accordionTopLevel = document.querySelector(".c-accordion-top-level");
  const categoryContainer = document.querySelector(".o-panel-categories");
  const accordionRect = accordionTopLevel.getBoundingClientRect();
  const accordionHeight = accordionRect.height + "px";
  const documentWidth = document.documentElement.clientWidth;

  if (documentWidth >= 980) {
    categoryContainer.style.height = accordionHeight;
  } else {
    categoryContainer.style.height = "auto";
  }
}

/**
 * @method onAccordionClicked
 * @description on accordion clicked
 * @param el {HTMLElement} c-control--accordion element
 * @param e {MouseEvent} click event
 * @return {void}
 */
export function onAccordionClicked(el, e) {
  const accordionItem = el.parentElement.parentElement;
  const subAccordionList = accordionItem.querySelector(accordionSelector);
  const buttonElement = el.querySelector(".c-control__button");
  const hasSubAccordion = subAccordionList !== null;
  const isOpen = hasSubAccordion && accordionItem.classList.contains(accordionOpenClass);

  // if clicking on a root accordion item, then close all other root level items
  if (isRootAccordionItem(accordionItem)) {
    selectedItem = accordionItem;
    closeAllItems();
  }

  if (hasSubAccordion && !isOpen) {
    accordionItem.classList.add(accordionOpenClass);
    buttonElement.setAttribute("aria-expanded", true);
  } else {
    accordionItem.classList.remove(accordionOpenClass);
    buttonElement.setAttribute("aria-expanded", false);
  }

  e.stopPropagation();

  ee.emit("accordion:clicked");
}
