import ee from "~/event-emitter";

const eventNamespace = "tgam-scroll-manager";

let layers = [];
let scrollPosition = null;

/**
 * @method registerLayer
 * @description registers a layer to keep track of how many elements are overlaid
 * @param object {object} an object containing an "isOpen" method and a "lockEvent" and "unlockEvent" property
 * @property object.isOpen {function} a method which determines whether the layer is enabled/visible on the page
 * @property object.key {string} a unique id to register and unregister layers
 * @property object.lockEvent {string} name of event that will initiate scroll lock (e.g. when layer has finished opening)
 * @property object.unlockEvent {string} name of event that will release scroll lock (e.g. when layer has finished closing)
 * @return {void}
 */
export function registerLayer(object) {
  layers.push(object);

  if (typeof object.key === "undefined") {
    console.error("scroll-manager: registerLayer method object parameter must have the 'key' property defined");
  }

  if (object.lockEvent) {
    ee.on(object.lockEvent, lockScrolling);
  }

  if (object.unlockEvent) {
    ee.on(object.unlockEvent, unlockScrolling);
  }

  if (object.resetScrollEvent) {
    ee.on(object.resetScrollEvent, resetScrollTop);
  }
}

/**
 * @method unregisterLayer
 * @description TODO
 * @param obj {object} an object with an "isOpen" method
 * @return {void}
 */
export function unregisterLayer(obj) {
  layers = layers.filter(layer => {
    if (layer.key === obj.key) {
      if (layer.lockEvent) {
        ee.off(layer.lockEvent, lockScrolling);
      }
      if (layer.unlockEvent) {
        ee.off(layer.unlockEvent, unlockScrolling);
      }
      if (layer.resetScrollEvent) {
        ee.off(layer.resetScrollEvent, resetScrollTop);
      }
      return false;
    }
    return true;
  });
}

/**
 * @method lockScrolling
 * @description makes the body unscrollable
 * @return {void}
 */
export function lockScrolling() {
  scrollPosition = document.body.scrollTop;
  document.body.classList.add("u-no-scroll");
  ee.emit(`${eventNamespace}:locked`, [true]);
}

/**
 * @method unlockScrolling
 * @description when a layer has finished closing, check whether it's safe to allow the body to scroll again (all layers must be closed)
 * @return {boolean}
 */
export function unlockScrolling() {
  resetScrollTop();
}

/**
 * @method resetScrollTop
 * @description removes no scroll class and reset scroll position
 * @return {void}
 */
export function resetScrollTop() {
  // every layer should have a method `isOpen` and the value for all of them must be `false`
  if (allLayersClosed()) {
    document.body.classList.remove("u-no-scroll");
    ee.emit(`${eventNamespace}:locked`, [false]);
    // reset to previous scroll top position
    document.body.scrollTop = scrollPosition;
  }
}

function allLayersClosed() {
  return layers.every((layer) => layer.isOpen && !layer.isOpen());
}
